Index: engine/influence.h
===================================================================
RCS file: /cvsroot/gnugo/gnugo/engine/influence.h,v
retrieving revision 1.16
diff -u -p -r1.16 influence.h
--- engine/influence.h	15 Feb 2003 14:32:54 -0000	1.16
+++ engine/influence.h	28 Apr 2003 13:33:45 -0000
@@ -27,6 +27,30 @@
 
 #include "liberty.h"
 
+
+/* STEPHANE NICOLET's modification : playing for influence in the fuseki */
+
+#define COSMIC_GNUGO 1
+
+
+#if COSMIC_GNUGO
+
+
+/* We use much more influence than the defaults attenuation coefficients !
+ * The "TERR_.."-values are used in the influence computations used
+ * for territory evaluation. (initial_influence with dragons_known,
+ * move_influence)
+ */
+#define DEFAULT_ATTENUATION      (fuseki_importance * 1.5 + (1.0 - fuseki_importance) * 3.0)
+#define TERR_DEFAULT_ATTENUATION (fuseki_importance * 1.2 + (1.0 - fuseki_importance) * 2.4)
+
+/* Extra damping coefficient for spreading influence diagonally. */
+#define DIAGONAL_DAMPING         (fuseki_importance * 1.2 + (1.0 - fuseki_importance) * 2.0)
+#define TERR_DIAGONAL_DAMPING    (fuseki_importance * 1.2 + (1.0 - fuseki_importance) * 1.7)
+
+
+#else
+
 /* Default attenuation coefficient.
  * The "TERR_.."-values are used in the influence computations used
  * for territory evaluation. (initial_influence with dragons_known,
@@ -39,6 +63,11 @@
 #define DIAGONAL_DAMPING 2.0
 #define TERR_DIAGONAL_DAMPING 1.7
 
+#endif
+
+
+
+
 
 /* Smallest amount of influence that we care about distributing. */
 #define INFLUENCE_CUTOFF 0.02
@@ -124,6 +153,34 @@ typedef int (*owner_function_ptr)(const 
 #define WEIGHT_TERRITORY 10
 #define WEIGHT_MOYO       3
 #define WEIGHT_AREA       1
+
+
+
+#if COSMIC_GNUGO
+
+/* fuseki_importance is a number between 0.0 and 1.0 ;
+ * when fuseki_importance is 0.0, the default influence
+ * values are used; when fuseki_importance is 1.0, GNU Go
+ * will try to play an influence-oriented fuseki by 
+ * over-estimatingthe potential territory values of moyos.
+ * In the current implementation, fuseki_importance decreases 
+ * slowly for 19*19 games from 1.0 at move 4 to 0.0 at move 100.
+ */
+float fuseki_importance;
+
+
+/* Used in the whose_moyo() function */
+struct moyo_determination_data
+{
+  float influence_balance;
+  float my_influence_minimum;
+  float opp_influence_maximum;
+};
+
+
+#endif
+
+
 
 /*
  * Local Variables:
Index: utils/gg_utils.c
===================================================================
RCS file: /cvsroot/gnugo/gnugo/utils/gg_utils.c,v
retrieving revision 1.30
diff -u -p -r1.30 gg_utils.c
--- utils/gg_utils.c	22 Feb 2003 10:54:24 -0000	1.30
+++ utils/gg_utils.c	28 Apr 2003 13:33:47 -0000
@@ -376,6 +376,9 @@ gg_sort(void *base, size_t nel, size_t w
   int gap = nel;
   int swap_made;
   char *end = (char *) base + width * (nel - 1);
+
+  if (nel <= 1)
+    return; /* Avoids invalid pointer comparison below for nel == 0 */
   do {
     char *a, *b;
     swap_made = 0;
Index: engine/influence.c
===================================================================
RCS file: /cvsroot/gnugo/gnugo/engine/influence.c,v
retrieving revision 1.78
diff -u -p -r1.78 influence.c
--- engine/influence.c	16 Apr 2003 22:03:43 -0000	1.78
+++ engine/influence.c	28 Apr 2003 13:33:45 -0000
@@ -72,6 +72,50 @@ static float followup_territory_cache[BO
 static int territory_cache_position_number = -1;
 static int territory_cache_color = -1;
 
+
+
+#if COSMIC_GNUGO
+
+/* Threholds values used in the whose_moyo() function */
+ struct moyo_determination_data moyo_data;
+ 
+/* Threholds value used in the whose_territory() function */
+ float territory_determination_value;
+ 
+#endif
+
+
+
+/* This curve determines how much influence is needed at least to claim
+ * an intersection as territory, in dependence of the "center value".
+ * (In the center, more effort is needed to get territory!)
+ * The center value is at the moment defined as follows:
+ * If d1, d2 are the distance to vertical and horizontal border, resp.,
+ * with d1<d2, then
+ * central = 3 * d1 + min(d2, 4)
+ * So this is mainly a function of the distance to the border; the
+ * distance to the second-nearest border gives a small correction of at
+ * most 4. This distinguishes edge and corner positions.
+ *
+ * The values for intersections close to a corner or to the edge have
+ * to be consistent such that standard corner enclosure etc. are
+ * sufficient to claim territory. The center values are more arbitrary
+ * suspect to tuning.
+ */
+
+static struct interpolation_data min_infl_for_territory =
+  { 6,  0.0, 24.0, { 6.0, 15.0, 26.0, 36.0, 45.0, 50.0, 55.0 }};
+
+/* Determines the territory correction factor in dependence of the ratio
+ * ( influence of stronger color / min_infl_for_territory(intersection))
+ */
+static struct interpolation_data territory_correction = 
+  { 5, (float) 0.0, 1.0, {0.0, 0.25, 0.45, 0.65, 0.85, 1.0}};
+
+
+
+
+
 /* If set, print influence map when computing this move. Purely for
  * debugging.
  */
@@ -331,6 +375,8 @@ accumulate_influence(struct influence_da
   }
 }
 
+
+
 /* Initialize the influence_data structure.  */
 
 static void
@@ -340,6 +386,48 @@ init_influence(struct influence_data *q,
 {
   int ii;
   float attenuation;
+  
+#if COSMIC_GNUGO
+  float t;
+    
+  /* Initialisation of some global positional values, based on 
+   * game stage. 
+   */
+    if ((board_size != 19) || (movenum <= 2))
+       fuseki_importance = 0.0;
+    else
+       {
+       /*
+       fuseki_importance = 1.0 - (movenum / 100.0)*(movenum / 100.0);
+       fuseki_importance = gg_max(0.0, fuseki_importance);
+       */
+       
+       fuseki_importance = 1.0;
+       }
+    /* 
+    printf("%f\n",fuseki_importance);
+    fprintf(stderr, "%f\n", fuseki_importance);
+    */
+
+    t = fuseki_importance;
+    
+    moyo_data.influence_balance     = t * 5.0  +  (1.0-t) * 7.0;
+    moyo_data.my_influence_minimum  = t * 5.0  +  (1.0-t) * 5.0;
+    moyo_data.opp_influence_maximum = t * 30.0 +  (1.0-t) * 10.0;
+    
+    territory_determination_value   = t * 0.85 +  (1.0-t) * 0.95;
+    
+    
+    min_infl_for_territory.values[0] = t * 6.0   +  (1.0-t) * 6.0;
+    min_infl_for_territory.values[1] = t * 15.0  +  (1.0-t) * 15.0;
+    min_infl_for_territory.values[2] = t * 25.0  +  (1.0-t) * 26.0;
+    min_infl_for_territory.values[3] = t * 25.0  +  (1.0-t) * 36.0;
+    min_infl_for_territory.values[4] = t * 25.0  +  (1.0-t) * 45.0;
+    min_infl_for_territory.values[5] = t * 25.0  +  (1.0-t) * 50.0;
+    min_infl_for_territory.values[6] = t * 25.0  +  (1.0-t) * 55.0;
+    
+#endif  
+
 
   if (q != &escape_influence) {
     q->color_to_move = color;
@@ -407,6 +495,12 @@ init_influence(struct influence_data *q,
 	 */
 	q->safe[ii] = 0;
     }
+    
+
+    
+    
+    
+    
 }
 
 /* Adds an influence source at position pos with prescribed strength
@@ -726,7 +820,35 @@ influence_callback(int anchor, int color
     pattern->autohelper(ll, pos, color, INFLUENCE_CALLBACK);
   }
   
-  
+#if COSMIC_GNUGO  
+
+  /* For I patterns, add a low intensity, both colored, influence
+   * source at *.
+   */
+  if (pattern->class & CLASS_I) {
+    int this_color = EMPTY;
+    float t = 1.0;
+    float source_strength = 0.1 * t * pattern->value;
+
+    if (q->color_to_move == EMPTY || (pattern->class & CLASS_s))
+      this_color = BLACK | WHITE;
+    else if (q->color_to_move != color)
+      this_color = q->color_to_move;
+      
+    /* Increase strength if we're computing escape influence. */
+    if (q == &escape_influence && (pattern->class & CLASS_e))
+      add_influence_source(pos, this_color, 20 * source_strength , 1.6, q);
+    else
+      add_influence_source(pos, this_color, source_strength, 1.6, q);
+
+    DEBUG(DEBUG_INFLUENCE,
+	  "  low intensity influence source at %1m, strength %f, color %C\n",
+	  pos, pattern->value, this_color);
+    return;
+  }
+
+#else
+
   /* For I patterns, add a low intensity, both colored, influence
    * source at *.
    */
@@ -749,6 +871,9 @@ influence_callback(int anchor, int color
 	  pos, pattern->value, this_color);
     return;
   }
+
+
+#endif
   
   /* For E patterns, add a new influence source of the same color and
    * pattern defined strength at *.
@@ -799,10 +924,26 @@ influence_callback(int anchor, int color
 	  
 	}
       }
+
+
+#if COSMIC_GNUGO
       
       /* Low intensity influence source for the color in turn to move. */
       if (pattern->class & CLASS_B) {
         if (q->is_territorial_influence)
+          enter_intrusion_source(anchor, ii, 0.15 * pattern->value,
+	  		         TERR_DEFAULT_ATTENUATION, q);
+        else
+          add_influence_source(ii, color,
+			       0.1 * pattern->value, DEFAULT_ATTENUATION, q);
+	DEBUG(DEBUG_INFLUENCE, "  intrusion at %1m\n", ii);
+      }
+      
+#else
+
+      /* Low intensity influence source for the color in turn to move. */
+      if (pattern->class & CLASS_B) {
+        if (q->is_territorial_influence)
           enter_intrusion_source(anchor, ii, pattern->value,
 	  		         TERR_DEFAULT_ATTENUATION, q);
         else
@@ -810,6 +951,10 @@ influence_callback(int anchor, int color
 			       pattern->value, DEFAULT_ATTENUATION, q);
 	DEBUG(DEBUG_INFLUENCE, "  intrusion at %1m\n", ii);
       }
+
+
+#endif
+      
     }
   }
 }
@@ -1027,12 +1172,27 @@ whose_territory(const struct influence_d
 
   ASSERT_ON_BOARD1(pos);
   
+  
+
+#if COSMIC_GNUGO
+
+
+  if (bi > 0.0 && wi == 0.0 && terr < -territory_determination_value)
+    return BLACK;
+
+  if (wi > 0.0 && bi == 0.0 && terr > territory_determination_value)
+    return WHITE;
+
+#else
+
   if (bi > 0.0 && wi == 0.0 && terr < -0.95)
     return BLACK;
 
   if (wi > 0.0 && bi == 0.0 && terr > 0.95)
     return WHITE;
 
+#endif
+
   return EMPTY;
 }
 
@@ -1049,12 +1209,31 @@ whose_moyo(const struct influence_data *
   int territory_color = whose_territory(q, pos);
   if (territory_color != EMPTY)
     return territory_color;
+    
+  
+#if COSMIC_GNUGO
+
+  if (bi > moyo_data.influence_balance * wi && 
+      bi > moyo_data.my_influence_minimum && 
+      wi < moyo_data.opp_influence_maximum)
+    return BLACK;
+  
+  if (wi > moyo_data.influence_balance * bi && 
+      wi > moyo_data.my_influence_minimum && 
+      bi < moyo_data.opp_influence_maximum)
+    return WHITE;
+
+
+#else
+
   
   if (bi > 7.0 * wi && bi > 5.0 && wi < 10.0)
     return BLACK;
 
   if (wi > 7.0 * bi && wi > 5.0 && bi < 10.0)
     return WHITE;
+
+#endif
   
   return EMPTY;
 }
@@ -1126,31 +1305,7 @@ whose_area(const struct influence_data *
   return EMPTY;
 }
 
-/* This curve determines how much influence is needed at least to claim
- * an intersection as territory, in dependence of the "center value".
- * (In the center, more effort is needed to get territory!)
- * The center value is at the moment defined as follows:
- * If d1, d2 are the distance to vertical and horizontal border, resp.,
- * with d1<d2, then
- * central = 3 * d1 + min(d2, 4)
- * So this is mainly a function of the distance to the border; the
- * distance to the second-nearest border gives a small correction of at
- * most 4. This distinguishes edge and corner positions.
- *
- * The values for intersections close to a corner or to the edge have
- * to be consistent such that standard corner enclosure etc. are
- * sufficient to claim territory. The center values are more arbitrary
- * suspect to tuning.
- */
-static struct interpolation_data min_infl_for_territory =
-  { 6,  0.0, 24.0, { 6.0, 15.0, 26.0, 36.0, 45.0, 50.0, 55.0 }};
-
-/* Determines the territory correction factor in dependence of the ratio
- * ( influence of stronger color / min_infl_for_territory(intersection))
- */
-static struct interpolation_data territory_correction = 
-  { 5, (float) 0.0, 1.0, {0.0, 0.25, 0.45, 0.65, 0.85, 1.0}};
-
+ 
 static void
 value_territory(struct influence_data *q)
 {
